// Generated by gmmproc 2.88.0 -- DO NOT MODIFY!


#include <glibmm.h>

#include <pangomm/fontmap.h>
#include <pangomm/private/fontmap_p.h>


/* Copyright 2001      The gtkmm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <https://www.gnu.org/licenses/>.
 */

#include <pangomm/cairofontmapimpl.h>
#include <pangomm/context.h>
#include <pango/pangocairo.h>

namespace Pango
{

// Custom wrap_new() because we want to create
// a CairoFontMapImpl if the underlying C class implements the PangoCairoFontMap interface.
Glib::ObjectBase* FontMap_Class::wrap_new(GObject* object)
{
  if (PANGO_IS_CAIRO_FONT_MAP(object))
     return new CairoFontMapImpl((PangoFontMap*)object);
  return new FontMap((PangoFontMap*)object);
}

std::vector<Glib::RefPtr<FontFamily>> FontMap::list_families() const
{
  //Get the array:
  PangoFontFamily** pFamilies = nullptr;
  int n_families = 0;
  pango_font_map_list_families(const_cast<PangoFontMap*>(gobj()), &pFamilies, &n_families);

  return Glib::ArrayHandler<Glib::RefPtr<FontFamily>>::array_to_vector
      (pFamilies, n_families, Glib::OWNERSHIP_SHALLOW);
}

} /* namespace Pango */

namespace
{
} // anonymous namespace


namespace Glib
{

Glib::RefPtr<Pango::FontMap> wrap(PangoFontMap* object, bool take_copy)
{
  return Glib::make_refptr_for_instance<Pango::FontMap>( dynamic_cast<Pango::FontMap*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Pango
{


/* The *_Class implementation: */

const Glib::Class& FontMap_Class::init()
{
  if(!gtype_) // create the GType if necessary
  {
    // Glib::Class has to know the class init function to clone custom types.
    class_init_func_ = &FontMap_Class::class_init_function;

    // This is actually just optimized away, apparently with no harm.
    // Make sure that the parent type has been created.
    //CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(pango_font_map_get_type());

    // Add derived versions of interfaces, if the C type implements any interfaces:
  Gio::ListModel::add_interface(get_type());

  }

  return *this;
}


void FontMap_Class::class_init_function(void* g_class, void* class_data)
{
  const auto klass = static_cast<BaseClassType*>(g_class);
  CppClassParent::class_init_function(klass, class_data);


}


/* The implementation: */

PangoFontMap* FontMap::gobj_copy()
{
  reference();
  return gobj();
}

FontMap::FontMap(const Glib::ConstructParams& construct_params)
:
  Glib::Object(construct_params)
{

}

FontMap::FontMap(PangoFontMap* castitem)
:
  Glib::Object((GObject*)(castitem))
{}


FontMap::FontMap(FontMap&& src) noexcept
: Glib::Object(std::move(src))
  , Gio::ListModel(std::move(src))
{}

FontMap& FontMap::operator=(FontMap&& src) noexcept
{
  Glib::Object::operator=(std::move(src));
  Gio::ListModel::operator=(std::move(src));
  return *this;
}


FontMap::~FontMap() noexcept
{}


FontMap::CppClassType FontMap::fontmap_class_; // initialize static member

GType FontMap::get_type()
{
  return fontmap_class_.init().get_type();
}


GType FontMap::get_base_type()
{
  return pango_font_map_get_type();
}


Glib::RefPtr<Font> FontMap::load_font(const Glib::RefPtr<Context>& context, const FontDescription& desc) const
{
  return Glib::wrap(pango_font_map_load_font(const_cast<PangoFontMap*>(gobj()), Glib::unwrap(context), (desc).gobj()));
}

Glib::RefPtr<Fontset> FontMap::load_fontset(const Glib::RefPtr<Context>& context, const FontDescription& desc, const Language& language) const
{
  return Glib::wrap(pango_font_map_load_fontset(const_cast<PangoFontMap*>(gobj()), Glib::unwrap(context), (desc).gobj(), const_cast<PangoLanguage*>((language).gobj())));
}

Glib::RefPtr<Context> FontMap::create_context()
{
  return Glib::wrap(pango_font_map_create_context(gobj()));
}

guint FontMap::get_serial() const
{
  return pango_font_map_get_serial(const_cast<PangoFontMap*>(gobj()));
}

Glib::RefPtr<FontFamily> FontMap::get_family(const Glib::ustring& name)
{
  auto retvalue = Glib::wrap(pango_font_map_get_family(gobj(), name.c_str()));
  if(retvalue)
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

Glib::RefPtr<const FontFamily> FontMap::get_family(const Glib::ustring& name) const
{
  return const_cast<FontMap*>(this)->get_family(name);
}

Glib::RefPtr<Font> FontMap::reload_font(const Glib::RefPtr<Font>& font, double scale, const Glib::RefPtr<Context>& context, const Glib::ustring& variations) const
{
  return Glib::wrap(pango_font_map_reload_font(const_cast<PangoFontMap*>(gobj()), Glib::unwrap(font), scale, Glib::unwrap(context), variations.empty() ? nullptr : variations.c_str()));
}

bool FontMap::add_font_file(const std::string& filename)
{
  GError* gerror = nullptr;
  auto retvalue = pango_font_map_add_font_file(gobj(), filename.c_str(), &(gerror));
  if(gerror)
    ::Glib::Error::throw_exception(gerror);
  return retvalue;
}


static_assert(Glib::Traits::ValueCompatibleWithWrapProperty<GType>::value,
  "Type GType cannot be used in _WRAP_PROPERTY. "
  "There is no suitable template specialization of Glib::Value<>.");

Glib::PropertyProxy_ReadOnly< GType > FontMap::property_item_type() const
{
  return Glib::PropertyProxy_ReadOnly< GType >(this, "item-type");
}

Glib::PropertyProxy_ReadOnly< unsigned int > FontMap::property_n_items() const
{
  return Glib::PropertyProxy_ReadOnly< unsigned int >(this, "n-items");
}


} // namespace Pango


